@@ -29,7 +29,7 @@ from flask import Response, request
 # NOTE the only missing should be "Flask"
 from flask import session, jsonify, Blueprint, make_response, abort, \
     redirect, url_for, after_this_request, send_file, send_from_directory, \
-    safe_join, escape, Markup, render_template, current_app, g
+    escape, Markup, render_template, current_app, g
 
 import logging
 log = logging.getLogger("fsa")
@@ -76,7 +76,7 @@ ANY, ALL, NONE = "ANY", "ALL", "NONE"
 _PREDEFS = (ANY, ALL, NONE)
 
 
-def typeof(p: inspect.Parameter):
+def _typeof(p: inspect.Parameter):
     """Guess parameter type, possibly with some type inference."""
     return dict if p.kind is inspect.Parameter.VAR_KEYWORD else \
         list if p.kind is inspect.Parameter.VAR_POSITIONAL else \
@@ -98,6 +98,9 @@ class Reference:
     r.whatever(…) # behaves as o.whatever(…)
     ```
 
+    The object may be thread-local or global depending on whether it is
+    initialized directly or by providing a generation functions.
+    The generation function is called on demand in each thread automatically.
     """
     class Local(object):
         pass
@@ -111,9 +114,9 @@ class Reference:
         """
         self._set(obj=obj, fun=fun, mandatory=False)
         if set_name and set_name != "_set":
-            setattr(self, set_name, getattr(self, "_set"))
-            setattr(self, set_name + "_obj", getattr(self, "_set_obj"))
-            setattr(self, set_name + "_fun", getattr(self, "_set_fun"))
+            setattr(self, set_name, self._set)
+            setattr(self, set_name + "_obj", self._set_obj)
+            setattr(self, set_name + "_fun", self._set_fun)
 
     def _set_obj(self, obj):
         """Set current wrapped object."""
@@ -225,7 +228,7 @@ _DIRECTIVES = {
     "FSA_PASSWORD_SCHEME", "FSA_PASSWORD_OPTS", "FSA_PASSWORD_LEN", "FSA_PASSWORD_RE",
     "FSA_HTTP_AUTH_OPTS",
     # internal caching
-    "FSA_CACHE", "FSA_CACHE_SIZE", "FSA_CACHE_OPTS",
+    "FSA_CACHE", "FSA_CACHE_SIZE", "FSA_CACHE_OPTS", "FSA_CACHE_PREFIX",
     # web-oriented settings
     "FSA_401_REDIRECT", "FSA_URL_NAME", "FSA_CORS", "FSA_CORS_OPTS",
 }
@@ -379,7 +382,7 @@ class FlaskSimpleAuth:
 
     def _params(self):
         """Get request parameters wherever they are."""
-        if request.json:
+        if request.is_json:
             return request.json
         else:
             # reimplement "request.values" after Flask 2.0 regression
@@ -394,7 +397,7 @@ class FlaskSimpleAuth:
         """Set `get_user_pass` helper, can be used as a decorator."""
         if self._get_user_pass:
             log.warning("overriding already defined get_user_pass hook")
-        self._get_user_pass = self._cache_function(gup, "g.")
+        self._get_user_pass = gup
         self._init_password_manager()
         return gup
 
@@ -402,7 +405,7 @@ class FlaskSimpleAuth:
         """Set `user_in_group` helper, can be used as a decorator."""
         if self._user_in_group:
             log.warning("overriding already defined user_in_group hook")
-        self._user_in_group = self._cache_function(uig, "u.")
+        self._user_in_group = uig
         return uig
 
     def cast(self, t, cast: Optional[Callable] = None):
@@ -505,9 +508,10 @@ class FlaskSimpleAuth:
         if cache:
             import cachetools as ct
             import CacheToolsUtils as ctu  # type: ignore
-            if cache in ("ttl", "lru", "lfu", "mru", "fifo", "rr"):
+            prefix = conf.get("FSA_CACHE_PREFIX", None)
+            if cache in ("ttl", "lru", "lfu", "mru", "fifo", "rr", "dict"):
                 maxsize = conf.get("FSA_CACHE_SIZE", _DEFAULT_CACHE_SIZE)
-                self._gen_cache = ctu.PrefixedCache
+                # build actual storage tier
                 if cache == "ttl":
                     ttl = self._cache_opts.pop("ttl", _DEFAULT_CACHE_TTL)
                     rcache: MutableMapping = ct.TTLCache(maxsize, **self._cache_opts, ttl=ttl)
@@ -521,23 +525,31 @@ class FlaskSimpleAuth:
                     rcache = ct.FIFOCache(maxsize, **self._cache_opts)
                 elif cache == "rr":
                     rcache = ct.RRCache(maxsize, **self._cache_opts)
+                elif cache == "dict":
+                    rcache = dict()
                 else:  # pragma: no cover
                     raise self._Bad(f"unexpected simple cache type: {cache}")
+                if prefix:
+                    rcache = ctu.PrefixedCache(rcache, prefix)
                 self._cache = ctu.StatsCache(rcache)
+                self._gen_cache = ctu.PrefixedCache
             elif cache in ("memcached", "pymemcache"):
                 import pymemcache as pmc  # type: ignore
                 if "serde" not in self._cache_opts:
                     self._cache_opts.update(serde=ctu.JsonSerde())
+                if prefix and "key_prefix" not in self._cache_opts:
+                    self._cache_opts.update(key_prefix=prefix.encode("utf-8"))
                 self._cache = ctu.StatsMemCached(pmc.Client(**self._cache_opts))
                 self._gen_cache = ctu.PrefixedMemCached
             elif cache == "redis":
                 import redis
                 ttl = self._cache_opts.pop("ttl", _DEFAULT_CACHE_TTL)
-                self._cache = ctu.StatsRedisCache(redis.Redis(**self._cache_opts), ttl=ttl)
+                rc = redis.Redis(**self._cache_opts)
+                if prefix:
+                    self._cache = ctu.PrefixedRedisCache(rc, prefix=prefix, ttl=ttl)
+                else:
+                    self._cache = ctu.RedisCache(rc, ttl=ttl)
                 self._gen_cache = ctu.PrefixedRedisCache
-            elif cache == "dict":
-                self._cache = ctu.StatsCache(dict())
-                self._gen_cache = ctu.PrefixedCache
             else:
                 raise self._Bad(f"unexpected FSA_CACHE: {cache}")
         #
@@ -744,12 +756,10 @@ class FlaskSimpleAuth:
     #
     def _get_fake_auth(self):
         """Return fake user. Only for local tests."""
-        assert request.remote_user is None, "do not shadow web server auth"
         assert request.remote_addr.startswith("127.") or \
                request.remote_addr == "::1", "fake auth only on localhost"
         params = self._params()
         user = params.get(self._login, None)
-        # it could check that the user exists in db
         if not user:
             raise self._Err("missing login parameter", 401)
         return user
@@ -828,12 +838,11 @@ class FlaskSimpleAuth:
     def _get_basic_auth(self):
         """Get user with basic authentication."""
         import base64 as b64
-        assert request.remote_user is None
         auth = request.headers.get("Authorization", None)
         if not auth:
             log.debug("AUTH (basic): missing authorization header")
             raise self._Err("missing authorization header", 401)
-        if auth[:6] != "Basic ":
+        if not auth.startswith("Basic "):
             log.debug(f"AUTH (basic): unexpected auth \"{auth}\"")
             raise self._Err("unexpected authorization header", 401)
         try:
@@ -854,7 +863,6 @@ class FlaskSimpleAuth:
     #
     def _get_param_auth(self):
         """Get user with parameter authentication."""
-        assert request.remote_user is None
         params = self._params()
         user = params.get(self._userp, None)
         if not user:
@@ -1088,42 +1096,24 @@ class FlaskSimpleAuth:
         "_check_object_perms": "p.",
     }
 
-    # NOTE for multiprocess setups:
-    # clearing a cache in a process does not tell others to do it as well…
-    def _cache_function(self, fun, prefix=None):
-        """Generate or regenerate cache for function."""
-        # get the actual function when regenerating caches
-        while hasattr(fun, "__wrapped__"):
-            fun = fun.__wrapped__
-        if not self._gen_cache:
-            return fun
-        else:
-            import cachetools as ct
-            cache = self._gen_cache(prefix=prefix, cache=self._cache)
-            return ct.cached(cache=cache)(fun)
-
     def _set_caches(self):
         """Create caches around some functions."""
-        for name, prefix in self._CACHABLE.items():
-            setattr(self, name, self._cache_function(getattr(self, name), prefix))
+        if self._gen_cache is not None and self._cache is not None:
+            log.debug(f"caching: {self._CACHABLE}")
+            import CacheToolsUtils as ctu
+            ctu.cacheMethods(cache=self._cache, obj=self, gen=self._gen_cache, **self._CACHABLE)
 
     def clear_caches(self):
-        """Clear internal cache. Probably a bad idea."""
+        """Clear internal shared cache.
+
+        Probably a bad idea because:
+        - of the performance impact
+        - for a local cache in a multi-process setup, other processes are out
+
+        The best option is to wait for cache entries to expire with a TTL.
+        """
         self._cache.clear()
 
-    def _safe_call(self, path, level, fun, *args, **kwargs):
-        """Call a route function ensuring a response whatever."""
-
-        try:  # the actual call
-            res = fun(*args, **kwargs)
-        except FSAException as e:  # something went wrong
-            res = self._Res(e.message, e.status)
-        except Exception as e:  # something went really wrong
-            log.error(f"internal error on {request.method} {request.path}: {e}")
-            res = self._Res(f"internal error caught at {level} on {path}", self._server_error)
-
-        return res
-
     #
     # INTERNAL DECORATORS
     #
@@ -1133,9 +1123,18 @@ class FlaskSimpleAuth:
     #    _perm_auth: check per-object permissions
     #   _any_noauth: validate that no authorization was needed
     #
+    def _safe_call(self, path, level, fun, *args, **kwargs):
+        """Call a route function ensuring a response whatever."""
+        try:  # the actual call
+            return fun(*args, **kwargs)
+        except FSAException as e:  # something went wrong
+            return self._Res(e.message, e.status)
+        except Exception as e:  # something went really wrong
+            log.error(f"internal error on {request.method} {request.path}: {e}")
+            return self._Res(f"internal error caught at {level} on {path}", self._server_error)
+
     def _authenticate(self, path, auth=None):
         """Decorator to authenticate current user."""
-
         # check auth parameter
         if auth:
             if isinstance(auth, str):
@@ -1229,7 +1228,7 @@ class FlaskSimpleAuth:
                 if n not in types and \
                    p.kind not in (p.VAR_KEYWORD, p.VAR_POSITIONAL):
                     # guess parameter type
-                    t = typeof(p)
+                    t = _typeof(p)
                     types[n] = t
                     typings[n] = self._casts.get(t, t)
                 if p.default != inspect._empty:  # type: ignore
@@ -1414,7 +1413,7 @@ class FlaskSimpleAuth:
             if i > 0:
                 spec, remainder = s.split(">", 1)
                 if ":" not in spec and spec in sig.parameters:
-                    t = typeof(sig.parameters[spec])
+                    t = _typeof(sig.parameters[spec])
                     # Flask supports 5 types, with string the default?
                     if t in (int, float, UUID, path):
                         splits[i] = f"{t.__name__.lower()}:{spec}>{remainder}"
